﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using Devart.Data.SQLite;
using Devart.Data.SQLite.Entity;
using Devart.Data.SQLite.Entity.Configuration;
using System.ComponentModel.DataAnnotations;

namespace SQLiteFullTextSearch {
  public class Book {

    [Key]
    [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
    [Column("rowid")]
    public int Rowid { get; set; }

    [MaxLength(200)]
    public string Author { get; set; }

    [MaxLength(300)]
    public string Name { get; set; }

    public string Content { get; set; }

    [Column("Books")]
    public string Fts { get; set; }

  }

  public class MyContext : DbContext {

    public MyContext() {
    }

    public MyContext(SQLiteConnection connection)
      : base(connection, false) {
    }

    static MyContext() {


    }
    public DbSet<Book> Books { get; set; }

    protected override void OnModelCreating(DbModelBuilder modelBuilder) {

    }

  }
}

namespace ConsoleApplication1
{
  class Program {
    static void Main(string[] args) {

      var monitor = new SQLiteMonitor() { IsActive = true };

      Database.SetInitializer<SQLiteFullTextSearch.MyContext>(null);

      using (var ctx = new SQLiteFullTextSearch.MyContext()) {

        // Load information into the Books table
        ctx.Books.Add(new SQLiteFullTextSearch.Book() {
          Author = "William Shakespeare",
          Name = "The Tragedy of King Richard the Second",
          Content = @"ACT 1
SCENE I. London. A Room in the palace.
[Enter KING RICHARD, attended; JOHN OF GAUNT, with other NOBLES.]
KING RICHARD.
  Old John of Gaunt, time-honoured Lancaster,
  Hast thou, according to thy oath and band,
  Brought hither Henry Hereford thy bold son,
  Here to make good the boisterous late appeal,
  Which then our leisure would not let us hear,
  Against the Duke of Norfolk, Thomas Mowbray?
..."
        });
        ctx.Books.Add(new SQLiteFullTextSearch.Book() {
          Author = "William Shakespeare",
          Name = "The Life and Death of King John",
          Content = @"ACT 1.
SCENE I. Northampton. A Room of State in the Palace.
[Enter KING JOHN, QUEEN ELINOR, PEMBROKE, ESSEX, SALISBURY, and others, with CHATILLON.]
KING JOHN.
  Now, say, Chatillon, what would France with us?
CHATILLON.
  Thus, after greeting, speaks the King of France,
  In my behaviour, to the majesty,
  The borrow'd majesty of England here.
..."
        });
        ctx.Books.Add(new SQLiteFullTextSearch.Book() {
          Author = "William Shakespeare",
          Name = "The Life of Henry the Fifth",
          Content = @"ACT 1.
SCENE I. London. An ante-chamber in the King's palace.
[Enter the Archbishop of Canterbury and the Bishop of Ely.]
CANTERBURY.
  My lord, I'll tell you: that self bill is urg'd,
  Which in the eleventh year of the last king's reign
  Was like, and had indeed against us pass'd,
  But that the scambling and unquiet time
  Did push it out of farther question.
..."
        });
        ctx.SaveChanges();

        // Write an analogue of the SQL query:
        //  SELECT rowid, name FROM books
        //  WHERE Books MATCH 'king palace Norfolk';
        var firstQuery = ctx.Books
          .Where(b => SQLiteTextFunctions.Match(b.Fts, "king palace Norfolk"))
          .Select(b => new { b.Rowid, b.Name });
        var firstQueryResults = firstQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("== Querying your single column full-text index ==");
        Console.WriteLine();
        Console.WriteLine("1) Natural language");
        Console.WriteLine();
        Console.WriteLine("SQL:");
        Console.WriteLine(firstQuery.ToString());
        Console.WriteLine();
        Console.WriteLine("Data:");
        Console.WriteLine("{0,-5} | {1}", "Id", "Name");
        Console.WriteLine(new string('-', 50));
        foreach (var item in firstQueryResults)
          Console.WriteLine("{0,-5} | {1} ", item.Rowid, item.Name);

        // Write an analogue of the SQL query:
        //   SELECT rowid, name FROM books
        //   WHERE Books MATCH '"king''s palace"'
        var secondQuery = ctx.Books
                  .Where(b => SQLiteTextFunctions.Match(b.Fts, "\"king's palace\""))
                  .Select(b => new { b.Rowid, b.Name });
        var secondQueryResults = secondQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("2) Phrase query");
        Console.WriteLine();
        Console.WriteLine("SQL:");
        Console.WriteLine(secondQuery.ToString());
        Console.WriteLine();
        Console.WriteLine("Data:");
        Console.WriteLine("{0,-5} | {1}", "Id", "Name");
        Console.WriteLine(new string('-', 50));
        foreach (var item in secondQueryResults)
          Console.WriteLine("{0,-5} | {1} ", item.Rowid, item.Name);

        Console.ReadLine();
      }
    }
  }
}